@@ -17,7 +17,7 @@ from scipy.integrate import odeint, ode, complex_ode
 #------------------------------------------------------------------------------
 
 
-class TestOdeint(object):
+class TestOdeint:
     # Check integrate.odeint
 
     def _do_problem(self, problem):
@@ -52,17 +52,19 @@ class TestOdeint(object):
             self._do_problem(problem)
 
 
-class TestODEClass(object):
+class TestODEClass:
 
     ode_class = None   # Set in subclass.
 
     def _do_problem(self, problem, integrator, method='adams'):
 
         # ode has callback arguments in different order than odeint
-        f = lambda t, z: problem.f(z, t)
+        def f(t, z):
+            return problem.f(z, t)
         jac = None
         if hasattr(problem, 'jac'):
-            jac = lambda t, z: problem.jac(z, t)
+            def jac(t, z):
+                return problem.jac(z, t)
 
         integrator_params = {}
         if problem.lband is not None or problem.uband is not None:
@@ -141,7 +143,8 @@ class TestOde(TestODEClass):
 
     def test_concurrent_fail(self):
         for sol in ('vode', 'zvode', 'lsoda'):
-            f = lambda t, y: 1.0
+            def f(t, y):
+                return 1.0
 
             r = ode(f).set_integrator(sol)
             r.set_initial_value(0, 0)
@@ -155,7 +158,8 @@ class TestOde(TestODEClass):
             assert_raises(RuntimeError, r.integrate, r.t + 0.1)
 
     def test_concurrent_ok(self):
-        f = lambda t, y: 1.0
+        def f(t, y):
+            return 1.0
 
         for k in range(3):
             for sol in ('vode', 'zvode', 'lsoda', 'dopri5', 'dop853'):
@@ -229,7 +233,7 @@ class TestComplexOde(TestODEClass):
             self._do_problem(problem, 'dop853')
 
 
-class TestSolout(object):
+class TestSolout:
     # Check integrate.ode correctly handles solout for dopri5 and dop853
     def _run_solout_test(self, integrator):
         # Check correct usage of solout
@@ -319,7 +323,7 @@ class TestSolout(object):
             self._run_solout_break_test(integrator)
 
 
-class TestComplexSolout(object):
+class TestComplexSolout:
     # Check integrate.ode correctly handles solout for dopri5 and dop853
     def _run_solout_test(self, integrator):
         # Check correct usage of solout
@@ -570,7 +574,7 @@ def jacv(t, x, omega):
     return j
 
 
-class ODECheckParameterUse(object):
+class ODECheckParameterUse:
     """Call an ode-class solver with several cases of parameter use."""
 
     # solver_name must be set before tests can be run with this class.
