@@ -7,7 +7,7 @@ import operator
 import numpy as np
 from scipy._lib._util import _prune_array
 
-from ._base import spmatrix, isspmatrix, SparseEfficiencyWarning
+from ._base import _spbase, issparse, SparseEfficiencyWarning
 from ._data import _data_matrix, _minmax_mixin
 from . import _sparsetools
 from ._sparsetools import (get_csr_submatrix, csr_sample_offsets, csr_todense,
@@ -15,8 +15,7 @@ from ._sparsetools import (get_csr_submatrix, csr_sample_offsets, csr_todense,
                            csr_column_index1, csr_column_index2)
 from ._index import IndexMixin
 from ._sputils import (upcast, upcast_char, to_native, isdense, isshape,
-                       getdtype, isscalarlike, isintlike, get_index_dtype,
-                       downcast_intp_index, get_sum_dtype, check_shape,
+                       getdtype, isscalarlike, isintlike, downcast_intp_index, get_sum_dtype, check_shape,
                        is_pydata_spmatrix)
 
 
@@ -26,7 +25,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
     def __init__(self, arg1, shape=None, dtype=None, copy=False):
         _data_matrix.__init__(self)
 
-        if isspmatrix(arg1):
+        if issparse(arg1):
             if arg1.format == self.format and copy:
                 arg1 = arg1.copy()
             else:
@@ -41,7 +40,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
                 M, N = self.shape
                 # Select index dtype large enough to pass array and
                 # scalar parameters to sparsetools
-                idx_dtype = get_index_dtype(maxval=max(M, N))
+                idx_dtype = self._get_index_dtype(maxval=max(M, N))
                 self.data = np.zeros(0, getdtype(dtype, default=float))
                 self.indices = np.zeros(0, idx_dtype)
                 self.indptr = np.zeros(self._swap((M, N))[0] + 1,
@@ -62,7 +61,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
                     maxval = None
                     if shape is not None:
                         maxval = max(shape)
-                    idx_dtype = get_index_dtype((indices, indptr),
+                    idx_dtype = self._get_index_dtype((indices, indptr),
                                                 maxval=maxval,
                                                 check_contents=True)
 
@@ -105,7 +104,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
 
         self.check_format(full_check=False)
 
-    def getnnz(self, axis=None):
+    def _getnnz(self, axis=None):
         if axis is None:
             return int(self.indptr[-1])
         else:
@@ -120,7 +119,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
                 return np.diff(self.indptr)
             raise ValueError('axis out of bounds')
 
-    getnnz.__doc__ = spmatrix.getnnz.__doc__
+    _getnnz.__doc__ = _spbase._getnnz.__doc__
 
     def _set_self(self, other, copy=False):
         """take the member variables of other and assign them to self"""
@@ -154,7 +153,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
             warn("indices array has non-integer dtype ({})"
                  "".format(self.indices.dtype.name), stacklevel=3)
 
-        idx_dtype = get_index_dtype((self.indptr, self.indices))
+        idx_dtype = self._get_index_dtype((self.indptr, self.indices))
         self.indptr = np.asarray(self.indptr, dtype=idx_dtype)
         self.indices = np.asarray(self.indices, dtype=idx_dtype)
         self.data = to_native(self.data)
@@ -205,7 +204,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
 
     def _scalar_binopt(self, other, op):
         """Scalar version of self._binopt, for cases in which no new nonzeros
-        are added. Produces a new spmatrix in canonical form.
+        are added. Produces a new sparse array in canonical form.
         """
         self.sum_duplicates()
         res = self._with_data(op(self.data, other), copy=True)
@@ -234,7 +233,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         elif is_pydata_spmatrix(other):
             return NotImplemented
         # Sparse other.
-        elif isspmatrix(other):
+        elif issparse(other):
             warn("Comparing sparse matrices using == is inefficient, try using"
                  " != instead.", SparseEfficiencyWarning, stacklevel=3)
             # TODO sparse broadcasting
@@ -272,7 +271,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         elif is_pydata_spmatrix(other):
             return NotImplemented
         # Sparse other.
-        elif isspmatrix(other):
+        elif issparse(other):
             # TODO sparse broadcasting
             if self.shape != other.shape:
                 return True
@@ -299,7 +298,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         elif isdense(other):
             return op(self.todense(), other)
         # Sparse other.
-        elif isspmatrix(other):
+        elif issparse(other):
             # TODO sparse broadcasting
             if self.shape != other.shape:
                 raise ValueError("inconsistent shapes")
@@ -370,7 +369,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         if isscalarlike(other):
             return self._mul_scalar(other)
         # Sparse matrix or vector.
-        if isspmatrix(other):
+        if issparse(other):
             if self.shape == other.shape:
                 other = self.__class__(other)
                 return self._binopt(other, '_elmul_')
@@ -511,7 +510,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         major_axis = self._swap((M, N))[0]
         other = self.__class__(other)  # convert to this format
 
-        idx_dtype = get_index_dtype((self.indptr, self.indices,
+        idx_dtype = self._get_index_dtype((self.indptr, self.indices,
                                      other.indptr, other.indices))
 
         fn = getattr(_sparsetools, self.format + '_matmat_maxnnz')
@@ -521,7 +520,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
                  np.asarray(other.indptr, dtype=idx_dtype),
                  np.asarray(other.indices, dtype=idx_dtype))
 
-        idx_dtype = get_index_dtype((self.indptr, self.indices,
+        idx_dtype = self._get_index_dtype((self.indptr, self.indices,
                                      other.indptr, other.indices),
                                     maxval=nnz)
 
@@ -551,7 +550,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
            self.data, y)
         return y
 
-    diagonal.__doc__ = spmatrix.diagonal.__doc__
+    diagonal.__doc__ = _spbase.diagonal.__doc__
 
     #####################
     # Other binary ops  #
@@ -575,7 +574,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
                 return mat
         elif isdense(other):
             return npop(self.todense(), other)
-        elif isspmatrix(other):
+        elif issparse(other):
             return self._binopt(other, op_name)
         else:
             raise ValueError("Operands not compatible.")
@@ -584,13 +583,13 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         return self._maximum_minimum(other, np.maximum,
                                      '_maximum_', lambda x: np.asarray(x) > 0)
 
-    maximum.__doc__ = spmatrix.maximum.__doc__
+    maximum.__doc__ = _spbase.maximum.__doc__
 
     def minimum(self, other):
         return self._maximum_minimum(other, np.minimum,
                                      '_minimum_', lambda x: np.asarray(x) < 0)
 
-    minimum.__doc__ = spmatrix.minimum.__doc__
+    minimum.__doc__ = _spbase.minimum.__doc__
 
     #####################
     # Reduce operations #
@@ -600,7 +599,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         """Sum the matrix over the given axis.  If the axis is None, sum
         over both rows and columns, returning a scalar.
         """
-        # The spmatrix base class already does axis=0 and axis=1 efficiently
+        # The _spbase base class already does axis=0 and axis=1 efficiently
         # so we only do the case axis=None here
         if (not hasattr(self, 'blocksize') and
                 axis in self._swap(((1, -1), (0, 2)))[0]):
@@ -618,12 +617,12 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
                 raise ValueError('dimensions do not match')
 
             return ret.sum(axis=(), dtype=dtype, out=out)
-        # spmatrix will handle the remaining situations when axis
+        # _spbase will handle the remaining situations when axis
         # is in {None, -1, 0, 1}
         else:
-            return spmatrix.sum(self, axis=axis, dtype=dtype, out=out)
+            return _spbase.sum(self, axis=axis, dtype=dtype, out=out)
 
-    sum.__doc__ = spmatrix.sum.__doc__
+    sum.__doc__ = _spbase.sum.__doc__
 
     def _minor_reduce(self, ufunc, data=None):
         """Reduce nonzeros with a ufunc over the minor axis when non-empty
@@ -695,7 +694,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         M = len(indices)
         new_shape = self._swap((M, N))
         if M == 0:
-            return self.__class__(new_shape)
+            return self.__class__(new_shape, dtype=self.dtype)
 
         row_nnz = self.indptr[indices + 1] - self.indptr[indices]
         idx_dtype = self.indices.dtype
@@ -722,7 +721,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         M = len(range(start, stop, step))
         new_shape = self._swap((M, N))
         if M == 0:
-            return self.__class__(new_shape)
+            return self.__class__(new_shape, dtype=self.dtype)
 
         # Work out what slices are needed for `row_nnz`
         # start,stop can be -1, only if step is negative
@@ -761,7 +760,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         k = len(idx)
         new_shape = self._swap((M, k))
         if k == 0:
-            return self.__class__(new_shape)
+            return self.__class__(new_shape, dtype=self.dtype)
 
         # pass 1: count idx entries and compute new indptr
         col_offsets = np.zeros(N, dtype=idx_dtype)
@@ -789,7 +788,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         start, stop, step = idx.indices(N)
         N = len(range(start, stop, step))
         if N == 0:
-            return self.__class__(self._swap((M, N)))
+            return self.__class__(self._swap((M, N)), dtype=self.dtype)
         if step == 1:
             return self._get_submatrix(minor=idx, copy=copy)
         # TODO: don't fall back to fancy indexing here
@@ -972,7 +971,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         do_sort = self.has_sorted_indices
 
         # Update index data type
-        idx_dtype = get_index_dtype((self.indices, self.indptr),
+        idx_dtype = self._get_index_dtype((self.indices, self.indptr),
                                     maxval=(self.indptr[-1] + x.size))
         self.indptr = np.asarray(self.indptr, dtype=idx_dtype)
         self.indices = np.asarray(self.indices, dtype=idx_dtype)
@@ -1043,7 +1042,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
             dtype=self.dtype
         )
 
-    tocoo.__doc__ = spmatrix.tocoo.__doc__
+    tocoo.__doc__ = _spbase.tocoo.__doc__
 
     def toarray(self, order=None, out=None):
         if out is None and order is None:
@@ -1062,7 +1061,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         csr_todense(M, N, x.indptr, x.indices, x.data, y)
         return out
 
-    toarray.__doc__ = spmatrix.toarray.__doc__
+    toarray.__doc__ = _spbase.toarray.__doc__
 
     ##############################################################
     # methods that examine or modify the internal data structure #
@@ -1187,8 +1186,8 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
             new_M, rm = divmod(shape[0], bm)
             new_N, rn = divmod(shape[1], bn)
             if rm or rn:
-                raise ValueError("shape must be divisible into %s blocks. "
-                                 "Got %s" % (self.blocksize, shape))
+                raise ValueError("shape must be divisible into {} blocks. "
+                                 "Got {}".format(self.blocksize, shape))
             M, N = self.shape[0] // bm, self.shape[1] // bn
         else:
             new_M, new_N = self._swap(shape)
@@ -1214,7 +1213,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
 
         self._shape = shape
 
-    resize.__doc__ = spmatrix.resize.__doc__
+    resize.__doc__ = _spbase.resize.__doc__
 
     ###################
     # utility methods #
@@ -1243,7 +1242,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         fn = getattr(_sparsetools, self.format + op + self.format)
 
         maxnnz = self.nnz + other.nnz
-        idx_dtype = get_index_dtype((self.indptr, self.indices,
+        idx_dtype = self._get_index_dtype((self.indptr, self.indices,
                                      other.indptr, other.indices),
                                     maxval=maxnnz)
         indptr = np.empty(self.indptr.shape, dtype=idx_dtype)
